#ifndef COMMON_H
#define COMMON_H

//    This file is part of Motion Watch.

//    Motion Watch is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    Motion Watch is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include <iostream>
#include <fstream>
#include <string>
#include <unistd.h>
#include <time.h>
#include <stdlib.h>
#include <errno.h>
#include <vector>
#include <thread>
#include <filesystem>
#include <mutex>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <opencv4/opencv2/opencv.hpp>
#include <opencv4/opencv2/videoio.hpp>

using namespace cv;
using namespace std;
using namespace std::filesystem;

#define APP_VER  "1.6.t9"
#define APP_NAME "Motion Watch"

struct shared_t
{
    vector<thread> detThreads;
    vector<string> conf;
    ofstream       recLogFile;
    ofstream       detLogFile;
    string         recLogPath;
    string         detLogPath;
    string         recordUrl;
    string         outDir;
    string         postCmd;
    string         buffDir;
    string         vidExt;
    string         vidCodec;
    string         camName;
    string         webBg;
    string         webTxt;
    string         webFont;
    string         webRoot;
    bool           init;
    bool           skipCmd;
    int            clipLen;
    int            frameGap;
    int            pixThresh;
    int            imgThresh;
    int            numOfClips;
    int            maxDays;
    int            maxClips;
    int            maxLogSize;
    int            retCode;
};

string         genDstFile(const string &dirOut, const char *fmt, const string &ext);
string         genTimeStr(const char *fmt);
string         cleanDir(const string &path);
string         parseForParam(const string &arg, int argc, char** argv, bool argOnly, int &offs);
string         parseForParam(const string &arg, int argc, char** argv, bool argOnly);
bool           createDir(const string &dir);
bool           createDirTree(const string &full_path);
void           enforceMaxDays(const string &dirPath, shared_t *share);
void           enforceMaxClips(const string &dirPath, shared_t *share);
void           rdLine(const string &param, const string &line, string *value);
void           rdLine(const string &param, const string &line, int *value);
void           statOut(shared_t *share);
void           waitForDetThreads(shared_t *share);
bool           rdConf(shared_t *share);
vector<string> parseForList(const string &arg, int argc, char** argv);
vector<string> lsFilesInDir(const string &path, const string &ext = string());
vector<string> lsDirsInDir(const string &path);

#endif // COMMON_H
